package de.fraunhofer.iem.dataprovider.debug.controller

import de.fraunhofer.iem.dataprovider.debug.dto.*
import de.fraunhofer.iem.dataprovider.gitlab.enumeration.PlatformEnum
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.taskManager.TaskManager
import de.fraunhofer.iem.dataprovider.taskManager.events.RecalculateAllKpisEvent
import de.fraunhofer.iem.dataprovider.toolRun.service.ToolRunService
import org.springframework.web.bind.annotation.*

@RestController
@RequestMapping("/debug/metrics")
class MetricsController(
    private val toolRunService: ToolRunService,
    private val taskManager: TaskManager
) {

    private val logger = getLogger(javaClass)

    @GetMapping("/latestRuns/{platform}/{id}")
    suspend fun repoChanged(@PathVariable platform: PlatformEnum, @PathVariable id: Long): ToolResultsDto {
        logger.info("Get latest tool runs for repo $id on platform $platform received.")
        val toolResults = toolRunService.getLatestToolRunsForRepo(id, platform)
        val repo = toolResults.first().repository!!
        val repoDto = RepoDto(repo.name!!, repo.url!!)
        val trDto = toolResults.map {
            ToolRunDto(
                it.findings.map { finding -> FindingDto(finding.message ?: "NO FINDING MESSAGE.") },
                ToolDto(it.tool?.name!!, it.tool?.version!!),
                it.createdAt!!
            )
        }
        return ToolResultsDto(repoDto, trDto)
    }

    @PostMapping("/recalculateAllKPIs")
    suspend fun recalculateAllKPIs() {
        taskManager.addEvent(RecalculateAllKpisEvent)
    }
}