package de.fraunhofer.iem.dataprovider.dependency.entity

import de.fraunhofer.iem.dataprovider.dependency.dto.DependencyCreateDto
import de.fraunhofer.iem.dataprovider.dependency.dto.VulnerabilityCreateDto
import de.fraunhofer.iem.dataprovider.dependency.dto.VulnerabilityScoreCreateDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import jakarta.persistence.*
import org.hibernate.Hibernate
import java.util.*

@Entity
@Table(name = "dependency")
class DependencyEntity {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @Column(name = "name", length = 600)
    var name: String? = null

    @OneToMany(
        mappedBy = "dependency",
        cascade = [CascadeType.ALL],
        orphanRemoval = true
    )
    var vulnerabilities: MutableSet<VulnerabilityEntity> = mutableSetOf()

    @ManyToMany(mappedBy = "dependencyEntities")
    var repositories: MutableSet<RepositoryEntity> = mutableSetOf()

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || Hibernate.getClass(this) != Hibernate.getClass(other)) return false
        other as DependencyEntity

        return id != null && id == other.id
    }

    override fun hashCode(): Int = javaClass.hashCode()

    fun toDto(): DependencyCreateDto {
        val dependencyCreateDto = DependencyCreateDto(this.name!!)
        this.vulnerabilities.forEach { vulnerabilityEntity ->
            val vulnerabilityCreateDto = VulnerabilityCreateDto(vulnerabilityEntity.cveIdentifier!!)

            vulnerabilityEntity.vulnerabilityScores.forEach { scoreEntity ->
                if (scoreEntity.severity != null && scoreEntity.scoringSystem != null) {
                    vulnerabilityCreateDto.vulnerabilityScores.add(
                        VulnerabilityScoreCreateDto(
                            scoreEntity.severity!!,
                            scoreEntity.scoringSystem!!
                        )
                    )
                }
            }

            dependencyCreateDto.vulnerabilities.add(vulnerabilityCreateDto)
        }
        return dependencyCreateDto
    }
}
