package de.fraunhofer.iem.dataprovider.repository.service

import de.fraunhofer.iem.dataprovider.gitlab.enumeration.PlatformEnum
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryCreateDto
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryDetailsCreateDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryDetailsEntity
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.repository.RepositoryDetailsRepository
import de.fraunhofer.iem.dataprovider.repository.repository.RepositoryRepository
import org.springframework.stereotype.Service
import java.util.*

@Service
class RepositoryService(
        private val repositoryRepository: RepositoryRepository,
        private val repositoryDetailsRepository: RepositoryDetailsRepository
) {
    private val logger = getLogger(javaClass)

    fun getRepoOrCreate(gitRepository: RepositoryCreateDto): RepositoryEntity {
        var repo = repositoryRepository.findByRepoIdAndPlatform(gitRepository.id, gitRepository.platform)
        if (repo == null) {
            repo = repositoryRepository.save(gitRepository.toDbObject())
        }
        return repo
    }

    // TODO: We should decide if we want to return optionals (null if not found) or throw exceptions or ...
    // TODO: Must be consistent across all methods
    fun findRepoByID(id: UUID): RepositoryEntity? {
        val repository = repositoryRepository.findById(id)
        if (repository.isEmpty) {
            return null
        }
        return repository.get()
    }

    fun save(repo: RepositoryEntity) {
        repositoryRepository.save(repo)
    }

    fun findRepoByRepoIdAndPlatform(repoId: Long, platform: PlatformEnum): RepositoryEntity? {
        return repositoryRepository.findByRepoIdAndPlatform(repoId, platform)
    }

    fun createRepositoryDetails(repositoryDetailsCreateDto: RepositoryDetailsCreateDto): RepositoryDetailsEntity {
        return repositoryDetailsRepository.save(repositoryDetailsCreateDto.toDbObject())
    }

    fun getLatestRepositoryDetailsByRepositoryId(repositoryId: UUID): RepositoryDetailsEntity? {
        val repositoryDetails = repositoryDetailsRepository.findFirstByRepository_IdOrderByCreatedAtDesc(repositoryId)
        if (repositoryDetails.isEmpty) {
            return null
        }
        return repositoryDetails.get()
    }

    fun getAllRepositories(): List<RepositoryEntity> {
        return repositoryRepository.findAll()
    }
}
