package de.fraunhofer.iem.dataprovider.taskManager

import de.fraunhofer.iem.dataprovider.taskManager.events.Event
import de.fraunhofer.iem.dataprovider.taskManager.events.GroupTasksDoneEvent
import de.fraunhofer.iem.dataprovider.taskManager.model.GroupTasks
import java.util.*

class GroupTaskManager(val responseChannel: suspend (event: Event) -> Unit) {

    private val dependentEvents: MutableMap<UUID, GroupTasks> = mutableMapOf()

    /**
     * Adds new group of tasks
     */
    fun createTaskGroup(repositoryId: UUID): UUID {
        val groupId = UUID.randomUUID()
        dependentEvents[groupId] = GroupTasks(repositoryId)
        return groupId
    }

    fun addTaskToGroup(groupId: UUID, taskId: UUID) {
        dependentEvents[groupId]?.taskIds?.add(taskId)
    }

    /**
     * Removes the finished task with the given ID from the internal task list for the
     * group. If the whole task group is finished a group finished event is sent.
     */
    suspend fun taskInGroupFinished(groupId: UUID, taskId: UUID) {
        val groupTasks = dependentEvents[groupId]
        if (groupTasks != null) {
            val taskGroupEmpty = removeTaskFromGroup(groupTasks, taskId)
            if (taskGroupEmpty) {
                removeGroup(groupId)
                responseChannel(GroupTasksDoneEvent(groupTasks.repositoryId))
            }
        }
    }

    private fun removeTaskFromGroup(groupTasks: GroupTasks, taskId: UUID): Boolean {
        groupTasks.taskIds.remove(taskId)
        return groupTasks.taskIds.isEmpty()
    }

    private fun removeGroup(groupId: UUID) {
        dependentEvents.remove(groupId)
    }
}
