package de.fraunhofer.iem.dataprovider.taskManager.tasks.dataQuery

import de.fraunhofer.iem.dataprovider.configuration.GitApiConfiguration
import de.fraunhofer.iem.dataprovider.gitlab.enumeration.PlatformEnum
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryCreateDto
import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
import de.fraunhofer.iem.dataprovider.taskManager.events.Event
import de.fraunhofer.iem.dataprovider.taskManager.events.GetGitlabProjectDoneEvent
import de.fraunhofer.iem.dataprovider.taskManager.tasks.Task
import org.gitlab4j.api.GitLabApi


class GetGitlabProjectTask(
    private val repoId: Long,
    private val gitlabConfiguration: GitApiConfiguration,
    override val responseChannel: suspend (event: Event) -> Unit,
    private val repositoryService: RepositoryService
) : Task() {


    private val gitlabApi: GitLabApi = GitLabApi(gitlabConfiguration.host, gitlabConfiguration.accessToken)

    override suspend fun execute() {
        logger.info("Gitlab config ${gitlabConfiguration.host} ${gitlabConfiguration.accessToken}")

        val project = gitlabApi.projectApi.getProject(repoId)
        logger.info(project.toString())
        val projectUri = project.httpUrlToRepo

        val gitRepository = RepositoryCreateDto(project.path, projectUri, repoId, PlatformEnum.OPEN_CODE)

        val repo = repositoryService.getRepoOrCreate(gitRepository)

        logger.info("Retrieved project ${project.path} and url $projectUri")

        responseChannel(GetGitlabProjectDoneEvent(taskID, repo.id!!, gitRepository))
    }
}
