package de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.occmd

import de.fraunhofer.iem.dataprovider.configuration.OpenCodeGitlabApiConfiguration
import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
import de.fraunhofer.iem.dataprovider.taskManager.events.Event
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.occmd.json.sarif.getOccmdSarifFromFilePath
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.SarifTask
import de.fraunhofer.iem.dataprovider.toolRun.service.ToolRunService
import org.springframework.core.io.ClassPathResource
import org.springframework.core.io.Resource
import java.io.File
import java.nio.file.Files
import java.nio.file.Path
import java.nio.file.Paths
import java.util.*

class OccmdTask(
    repositoryDirectoryPath: String,
    outputDirectoryPath: String,
    openCodeGitlabApiConfiguration: OpenCodeGitlabApiConfiguration,
    private val repositoryService: RepositoryService,
    override val responseChannel: suspend (task: Event) -> Unit,
    override val repoId: UUID,
    override val groupID: UUID? = null,
    override val toolRunService: ToolRunService
) : SarifTask() {

    private val resource: Resource = ClassPathResource("scripts/occmd.sh")

    override val outputDirectory: String = Paths.get(outputDirectoryPath, "occmd", taskID.toString()).toString()
    private val copiedRepositoryPath = Paths.get(outputDirectory, "copiedRepo")
    override val getSarif = ::getOccmdSarifFromFilePath

    private val repositoryID = getRepositoryId(repoId)
    private val checkSecretPath =
        Paths.get(outputDirectory, "app", "notes", "opencode.d", "code", "resources", "checks", "secrets")

    override val flags: Array<String> = arrayOf(
        resource.file.absolutePath,
        "--key",
        openCodeGitlabApiConfiguration.accessToken,
        "--username",
        "USERNAME", //TODO: Make sure is it required? if not delete otherwise dynamically take the username
        "--proj-path",
        copiedRepositoryPath.toString(),
        "--out-dir",
        Paths.get(outputDirectory, "app", "notes").toAbsolutePath().toString(),
        "--console-out-file",
        Paths.get(outputDirectory, "console.txt").toAbsolutePath().toString(),
        "--proj-id",
        repositoryID
    )

    override val resultFileOutputPath: Path =
        Paths.get(checkSecretPath.toAbsolutePath().toString(), repositoryID, "DetectSecrets.baseline")

    private fun getRepositoryId(repoId: UUID): String {
        val repoPlatformId = repositoryService.findRepoByID(repoId)?.repoId
            ?: throw NoSuchElementException("Repository with id $repoId was not found.")
        return repoPlatformId.toString()
    }


    private val dbRawPath = Paths.get(outputDirectory, "app", "notes", "opencode.d", "code", "db", "raw")
    private val blacklistPath = Paths.get(
        outputDirectory,
        "app",
        "notes",
        "opencode.d",
        "code",
        "resources",
        "checks",
        "checked_in_binaries",
        "blacklist"
    )

    override fun cleanUp() {
        File(copiedRepositoryPath.toUri()).deleteRecursively()
    }

    init {
        val repoFile = File(repositoryDirectoryPath)
        val copiedRepo = File(copiedRepositoryPath.toUri())
        repoFile.copyRecursively(copiedRepo)

        Files.createDirectories(dbRawPath)
        Files.createDirectories(blacklistPath)
        Files.createDirectories(checkSecretPath)
    }
}
