package de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.occmd.json.sarif

import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.occmd.json.raw.OccmdRaw
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.json.*
import kotlinx.serialization.json.Json
import java.io.IOException
import java.nio.file.Path


fun getOccmdSarifFromFilePath(resultPath: Path): SarifJson {
    val resFile = resultPath.toFile()
    if (resFile.exists()) {
        val resString = resFile.readText()
        val json = Json { ignoreUnknownKeys = true}
        val occmdRaw = json.decodeFromString<OccmdRaw>(resString)

        return occmdRawToSarif(occmdRaw)
    }

    throw IOException("File not found / Path is no file.")
}

private fun occmdRawToSarif(occmdRaw: OccmdRaw): SarifJson {
    val tool = SarifToolJson(SarifDriverJson(name = "OCCMD", fullName = "OCCMD", version = occmdRaw.version))

    val results: MutableList<SarifResultJson> = mutableListOf()

    for (elem in occmdRaw.results) {
        for (rawResult in elem.value) {
            val location = SarifLocationJson(
                SarifPhysicalLocationJson(
                    SarifArtifactLocationJson(rawResult.filename),
                    SarifRegionJson(startLine = rawResult.line_number)
                )
            )

            val message = SarifMessageJson("hashed_string: ${rawResult.hashed_secret} and is_verified: ${rawResult.is_verified}")
            val result = SarifResultJson("warning", listOf(location), message, rawResult.type)

            results.add(result)
        }
    }

    val run = SarifRunJson(tool = tool, results = results)
    return SarifJson(listOf(run))
}
