#!/bin/sh

OC_API_KEY=${1}
OC_USERNAME=${2}
PROJ_PATH=${3}
OUT_DIR=${4}
CONSOLE_OUT_FILE=${5}
PROJ_ID=${6}

OC_API_KEY_FLAG=false
OC_USERNAME_FLAG=false
PROJ_PATH_FLAG=false
OUT_DIR_FLAG=$false
CONSOLE_OUT_FILE_FLAG=false
PROJ_ID_FLAG=false

for arg in "$@"; do
  shift
  case "$arg" in
    '--key') 
      set -- "$@" '-k';;
    '--username') 
      set -- "$@" '-u';;
    '--proj-path') 
      set -- "$@" '-p';;
    '--out-dir') 
      set -- "$@" '-o';;
    '--console-out-file') 
      set -- "$@" '-c';;
    '--proj-id') 
      set -- "$@" '-i';;
    *) 
      set -- "$@" "$arg";;
  esac
done

while getopts k:u:p:o:c:i: option 
do
  case "${option}"
  in
    k)
      OC_API_KEY_FLAG=true
      OC_API_KEY=${OPTARG};;
    u) 
      OC_USERNAME_FLAG=true
      OC_USERNAME=${OPTARG};;
    p) 
      PROJ_PATH_FLAG=true
      PROJ_PATH=${OPTARG};;
    o) 
      OUT_DIR_FLAG=true
      OUT_DIR=${OPTARG};;
    c) 
      CONSOLE_OUT_FILE_FLAG=true
      CONSOLE_OUT_FILE=${OPTARG};;
    i) 
      PROJ_ID_FLAG=true
      PROJ_ID=${OPTARG};;
  esac
done

if [ "$OC_API_KEY_FLAG" != true ]
then 
  echo "API key is not given"
  exit 1
fi

if [ "$OC_USERNAME_FLAG" != true ]
then 
  echo "Username is not given"
  exit 1
fi

if [ "$PROJ_PATH_FLAG" != true ]
then 
  echo "path of the project being analyzed is not given"
  exit 1
fi

if [ "$OUT_DIR_FLAG" != true ]
then 
  echo "Path of the output directory is not given"
  exit 1
fi

if [ "$CONSOLE_OUT_FILE_FLAG" != true ]
then 
  echo "Path of the console output file is not given"
  exit 1
fi

if [ "$PROJ_ID_FLAG" != true ]
then 
  echo "project ID is not given"
  exit 1
fi

echo $OC_API_KEY
echo $OC_USERNAME
echo $PROJ_PATH
echo $OUT_DIR
echo $CONSOLE_OUT_FILE
echo $PROJ_ID

docker run \
--rm \
-e OC_GL_APIKEY=${OC_API_KEY} \
-e OC_GL_USER=${OC_USERNAME} \
-v ${PROJ_PATH}:/app/proj/ \
-v ${OUT_DIR}:/app/notes/ \
occmd \
python3 occmd check --check Secrets --id ${PROJ_ID} --directory /app/proj/ > ${CONSOLE_OUT_FILE}